/*******************************************************************************
* Copyright 2007 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
using System;
using System.Collections;
using System.IO;
using System.Text;
//using JobSync.TTSoap;

namespace JobSync
{
   public class CGIEngine
   {
      protected string m_strUsername, m_strPassword;
      protected ttsoapcgi m_cgiEngine = null;
      protected long m_lSession = 0;
      protected string m_strLastError = "";

      public CGIEngine(string p_strSoapCGIUrl, string p_strUsername, string p_strPassword)
		{
         m_cgiEngine = new ttsoapcgi();
         m_cgiEngine.Url = p_strSoapCGIUrl;
         m_strUsername = p_strUsername;
         m_strPassword = p_strPassword;
		}

      public bool updateDefect(string p_strProject, string p_strDefectNum, ArrayList p_listFiles)
      {
         bool fResult = false;
         try
         {
            if (login(p_strProject, m_strUsername, m_strPassword))
            {
               CDefect defect = m_cgiEngine.editDefect( m_lSession, Convert.ToInt32(p_strDefectNum), "", false);
               if (defect == null) { return (false); }

               ArrayList listRecs = new ArrayList();
               CSCCFileRecord[] afiles = defect.pSCCFileList;
               for (int i = 0; afiles != null && i < afiles.Length; ++i) { listRecs.Add(afiles[i]); }

               // No intelligent appending here, just brute force add.
               for (int i = 0; i < p_listFiles.Count; ++i)
               {
                  P4Change p4 = (P4Change)p_listFiles[i];
                  CSCCFileRecord rec = new CSCCFileRecord();
                  rec.mstrFileName = p4.m_strFile;
                  rec.mstrFixedRevision = p4.m_strRev + " [c" + p4.m_strChange + "]";
                  listRecs.Add(rec);
               }

               afiles = new CSCCFileRecord[listRecs.Count];
               for (int i = 0; i < listRecs.Count; ++i) { afiles[i] = (CSCCFileRecord)listRecs[i]; }
               defect.pSCCFileList = afiles;
               m_cgiEngine.saveDefect(m_lSession, defect);
               fResult = true;
            }
         }
         catch (Exception p_exception)
         {
            m_strLastError = "addDefect Failed: " + p_exception.Message;
         }
         finally
         {
            logout();
         }

         return (fResult);
      }

      public string lastError() { return (m_strLastError); }

      protected bool login(string p_strProject, string p_strUsername, string p_strPassword)
      {
         try
         {
            // -harpm. Let's just pull from TTP's list and match-up.
            CProject project = null;
            CProject[] aproject = m_cgiEngine.getProjectList( p_strUsername, p_strPassword);
            for (int i = 0; i < aproject.Length && project == null; ++i)
            {
               if (aproject[i].database.name == p_strProject) { project = aproject[i]; }
            }

            m_lSession = m_cgiEngine.ProjectLogon(project, p_strUsername, p_strPassword);
         }
         catch (Exception p_exception)
         {
            m_strLastError = "ProjectLogon: " + p_exception.Message;
            m_lSession = 0;
         }
         return (m_lSession > 0);
      }

      protected bool logout()
      {
         try
         {
            if (m_lSession > 0)
               m_cgiEngine.DatabaseLogoff(m_lSession);
         }
         catch (Exception p_exception)
         {
            m_strLastError = "DatabaseLogoff: " + p_exception.Message;
         }
         finally
         {
            m_lSession = 0;
         }
         return (true);
      }
   }
}
