/*******************************************************************************
* Copyright 2007 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
using System;
using System.Collections;
using System.Text;
using System.Text.RegularExpressions;
using P4COM;
using SPSTools;

namespace JobSync
{
   class CreateJob
   {
      protected Logger m_logger = null;
      protected p4 m_p4api = null;
      protected ArrayList m_listFormFields = new ArrayList();

      public CreateJob(Logger p_logger)
      {
         m_logger = p_logger;
         m_p4api = new p4();
      }

      public bool run(Hashtable p_hash)
      {
         bool fResult = false;
         try
         {
            if (!buildFormFields()) { return (false); }

            m_p4api.ParseForms(); // MUST be set before the connect, allows us to work with forms.
            m_p4api.Connect();
            m_p4api.ExceptionLevel = 1; // 0 - No exceptions, 1 - Exception on error, 2 - Exception on error or warning.

            // Fetch the job description.
            Array aJobSpec = m_p4api.run("job -o"); // Give us the jobspec, in pre-canned format.

            for (int i = 0; i < m_listFormFields.Count; ++i)
            {  // Parsing here leans heavily on the default Job form spec.
               string strVal = m_p4api.get_Var((string)m_listFormFields[i]);
               if (strVal == null) { continue; } // non-string fields and optional fields not populated in the form.

               strVal = strVal.Trim().TrimEnd('\n');
               if (strVal == "\"\"" || (strVal.StartsWith("<") && strVal.EndsWith(">")))
               {  // Making an assumption here as to format of fields that need to be filled out.
                  switch ((string)m_listFormFields[i])
                  {
                     case "DefectNum":
                        m_p4api.set_Var( (string)m_listFormFields[i], (string)p_hash["Number"]);
                        break;
                     case "Description":
                        m_p4api.set_Var( (string)m_listFormFields[i], "Summary-" + (string)p_hash["Summary"] + "\n\nDescription: \n" + (string)p_hash["Description"]);
                        break;
                  }
               }
            }

            Array aResult = m_p4api.run("job -i");
            m_logger.log("Success: " + (string)aResult.GetValue(0));
            fResult = true;
         }
         catch (Exception p_exception)
         {
            m_logger.log("CreateJob failed: " + p_exception.Message);
         }
         finally { m_p4api.Disconnect(); }
         return (fResult);
      }

      protected bool buildFormFields()
      {
         bool fResult = false;
         try
         {
            m_p4api.Connect();
            m_p4api.ExceptionLevel = 1; // 0 - No exceptions, 1 - Exception on error, 2 - Exception on error or warning.

            // Fetch the job description.
            Array aJobSpec = m_p4api.run("jobspec -o"); // Give us the jobspec, in pre-canned format.
            string[] astr = ((string)aJobSpec.GetValue(0)).Split(new string[] { "\n\n" }, new StringSplitOptions());

            // You could narrow down the result set here to only field name, or use the rest
            // of the field's information to add more intelligence when creating the job.
            Regex regex = new Regex(@"([0-9])+[ ](?<rslt>.+)");
            for (int i = 0; i < astr.Length; ++i)
            {
               if (astr[i].IndexOf("Fields:") >= 0)
               {
                  MatchCollection m = regex.Matches(astr[i]);
                  for (int j = 0; j < m.Count; ++j)
                     m_listFormFields.Add( m[j].Groups["rslt"].Value.Split(' ')[0]);
               }
            }

            fResult = (m_listFormFields.Count > 0);
         }
         catch (Exception p_exception)
         {
            m_logger.log("Unable to fetch job specification, creation will fail: " + p_exception.Message);
         }
         finally { m_p4api.Disconnect(); }
         return (fResult);
      }
   }
}
