/*******************************************************************************
* Copyright 2007 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
using System;
using System.Collections;
using System.Text;
using SPSTools;

namespace JobSync
{
   class JobSync
   {
      static Hashtable buildValueHash()
      {  // This is just a quick hack, for real use you'd want to do some kind of mapping between 
         // P4 field name and environment variables. That would make extending this easier, and
         // require just one config file to change functionality from.
         Hashtable hash = new Hashtable();
         try
         {
            // Flip the comments here to test this w/o a TT trigger.
            hash.Add("Project", Environment.GetEnvironmentVariable("PROJNAME"));
//            hash.Add("Project", "CreateDefect");
            hash.Add("Number", Environment.GetEnvironmentVariable("DNUM"));
//            hash.Add("Number", "21");
            hash.Add("Summary", Environment.GetEnvironmentVariable("SUMM"));
//            hash.Add("Summary", "My new defect!");
            hash.Add("Description", Environment.GetEnvironmentVariable("DESC"));
//            hash.Add("Description", "My defect description!");
         }
         catch (Exception p_exception)
         {
            Console.WriteLine("Building value table failed: " + p_exception.Message);
         }
         return (hash);
      }

      static void Main(string[] p_args)
      {
         try
         {
            Logger logger = new Logger("C:\\P4JobSync.log");

            if (p_args.Length == 4 && p_args[0] == "ttsync")
            {
               SyncTTP sync = new SyncTTP(logger, p_args[1], p_args[2], p_args[3]);
               sync.run(buildValueHash());
            }
            if (p_args.Length == 6 && p_args[0] == "p4sync")
            {
               SyncFromP4 sync = new SyncFromP4(logger, p_args[1], p_args[2], p_args[3], p_args[5]);
//               Hashtable hash = buildValueHash();
//               hash["Project"] = p_args[4];
               sync.run(p_args[4]);
            }
            else if (p_args.Length >= 1 && p_args[0] == "create")
            {
               CreateJob job = new CreateJob(logger);
               job.run(buildValueHash());
            }
            else
            {
               string str = "You have passed incorrect cmd-line parameters. Supported parameters, their format and ordering are as follows:\n";
               str += "\n\t<Operation> ('create' | 'ttsync' | 'p4sync')";
               str += "\n\t<TT SDK Url> (ignored if operation=create)";
               str += "\n\t<TT Username> (ignored if operation=create)";
               str += "\n\t<TT Password> (ignored if operation=create)";
               str += "\n\t<TT Project> (ignored if operation!=p4sync)";
               str += "\n\t<P4 Changelist #> (ignored if operation!=p4sync)";
               str += "\n\nFor example:\n";
               str += "\n\tC:\\app.exe create http://localhost/scripts/ttsoapcgi.exe myuser mypassword";
               logger.log(str);
            }
         }
         catch (Exception p_exception) { Console.WriteLine("App Crashed: " + p_exception.Message); }
      }
   }
}
