/*******************************************************************************
* Copyright 2007 Seapine Software, Inc. All rights reserved.
*
* IMPORTANT:  This Software is supplied to you by Seapine Software, Inc. (Seapine)
* in consideration of your agreement to the following terms, and your use, installation,
* modification or redistribution of this Software constitutes acceptance of these terms.
* If you do not agree with these terms, please do not use, install, modify or redistribute
* this Software.
*
* In consideration of your agreement to abide by the following terms, and subject to these
* terms, Seapine grants you a personal, non-exclusive license, under Seapine copyrights in
* this original Seapine software (the Software), to use, reproduce, modify and redistribute
* the Software, with or without modifications, in source and/or binary forms; provided that
* if you redistribute the Software in its entirety and without modifications, you must retain
* this notice and the following text and disclaimers in all such redistributions of the Software.
* Neither the name, trademarks, service marks or logos of Seapine, Inc. may be used to endorse
* or promote products derived from the Software without specific prior written permission from
* Seapine.  Except as expressly stated in this notice, no other rights or licenses, express or
* implied, are granted by Seapine herein, including but not limited to any patent rights that
* may be infringed by your derivative works or by other works in which the Software may be
* incorporated.
*
* The Software is provided by Seapine on an "AS IS" basis.  SEAPINE MAKES NO WARRANTIES,
* EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT,
* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE SOFTWARE OR ITS USE AND
* OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
*
* IN NO EVENT SHALL SEAPINE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE,
* REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE SOFTWARE, HOWEVER CAUSED AND WHETHER
* UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN
* IF SEAPINE HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
using System;
using System.Collections;
using System.Text.RegularExpressions;
using P4COM;
using SPSTools;

namespace JobSync
{
   // Quick & dirty, to simplify the hand-off to Soap.
   class P4Change { public string m_strChange; public string m_strFile; public string m_strRev; public P4Change(string p_strChange, string p_strFile, string p_strRev) { m_strChange = p_strChange; m_strFile = p_strFile; m_strRev = p_strRev; } }

   class SyncTTP
   {
      protected p4 m_p4api = null;
      protected CGIEngine m_engine = null;
      protected Logger m_logger = null;

      public SyncTTP(Logger p_logger, string p_strTTUrl, string p_strUsername, string p_strPassword)
      {
         m_logger = p_logger;
         m_p4api = new p4();
         m_engine = new CGIEngine(p_strTTUrl, p_strUsername, p_strPassword);
      }

      public bool run(Hashtable p_hash)
      {
         bool fResult = false;
         try
         {
            ArrayList list = getFiles(getChanges((string)p_hash["Number"]));
            fResult = m_engine.updateDefect((string)p_hash["Project"], (string)p_hash["Number"], list);
            m_logger.log("Defect " + (string)p_hash["Number"] + " updated successfully");
         }
         catch (Exception p_exception)
         {
            m_logger.log("SyncTTP Failed: " + p_exception.Message);
         }
         finally { m_p4api.Disconnect(); }
         return (fResult);
      }

      protected ArrayList getFiles(ArrayList p_listChanges)
      {
         ArrayList list = new ArrayList();
         try
         {
            m_p4api.Tagged();
            m_p4api.Connect();
            m_p4api.ExceptionLevel = 1; // 0 - No exceptions, 1 - Exception on error, 2 - Exception on error or warning.

            for (int i = 0; i < p_listChanges.Count; ++i)
            {
               Array aFiles = m_p4api.run("describe " + (string)p_listChanges[i]);
               string strFile = "";
               for (int j = 0; j < aFiles.Length; ++j)
               {
                  if (((string)aFiles.GetValue(j)).StartsWith("depotFile"))
                  {
                     if (strFile.Length > 0) { list.Add(new P4Change((string)p_listChanges[i], strFile, "?")); }
                     strFile = ((string)aFiles.GetValue(j));
                     int iPos = strFile.IndexOf(' ');
                     if (iPos >= 0) { strFile = strFile.Substring(iPos + 1, strFile.Length - iPos - 1); }
                  }
                  else if (strFile.Length > 0 && ((string)aFiles.GetValue(j)).StartsWith("rev"))
                  {
                     string strRev = ((string)aFiles.GetValue(j));
                     int iPos = strRev.IndexOf(' ');
                     if (iPos >= 0) { strRev = strRev.Substring(iPos + 1, strRev.Length - iPos - 1); }
                     list.Add(new P4Change((string)p_listChanges[i], strFile, strRev));
                     strFile = "";
                  }
               }
            }
         }
         catch (Exception p_exception) { m_logger.log("File fetch failed: " + p_exception.Message); }
         finally { m_p4api.Disconnect(); }
         return (list);
      }

      protected ArrayList getChanges(string p_strDefectNum)
      {
         ArrayList list = new ArrayList();
         try
         {
            Regex regex = new Regex("change[ ][0-9]+[ ]");

            m_p4api.Connect();
            m_p4api.ExceptionLevel = 1; // 0 - No exceptions, 1 - Exception on error, 2 - Exception on error or warning.

            Array aJobs = m_p4api.run("jobs -e DefectNum=" + p_strDefectNum);
            for (int i = 0; i < aJobs.Length; ++i)
            {
               Array aChanges = m_p4api.run("fixes -j " + ((string)aJobs.GetValue(i)).Split(' ')[0]);
               for (int j = 0; j < aChanges.Length; ++j)
               {
                  Match m = regex.Match((string)aChanges.GetValue(j));
                  list.Add(((string)m.Value).Replace("change ", ""));
               }
            }
         }
         catch (Exception p_exception) { m_logger.log("Changelist fetch failed: " + p_exception.Message); }
         finally { m_p4api.Disconnect(); }
         return (list);
      }
   }
}
